@@ -175,71 +175,132 @@ make_directory_with_parents (GFile         *file,
   return g_file_make_directory (file, cancellable, error);
 }
 
-static gchar*
-strip_characters (const gchar *original)
+static gboolean
+strip_find_next_block (const gchar    *original,
+		       const gunichar  open_char,
+		       const gunichar  close_char,
+		       gint           *open_pos,
+		       gint           *close_pos)
 {
-	const gchar *foo = "()[]<>{}_!@#$^&*+=|\\/\"'?~";
-	guint osize = strlen (original);
-	gchar *retval = (gchar *) g_malloc0 (sizeof (gchar *) * osize + 1);
-	guint i = 0, y = 0;
+	const gchar *p1, *p2;
 
-	while (i < osize) {
-		/* Remove (anything) */
-		if (original[i] == '(') {
-			gchar *loc = strchr (original+i, ')');
-			if (loc) {
-				i = loc - original + 1;
-				continue;
-			}
-		}
-
-		/* Remove [anything] */
-		if (original[i] == '[') {
-			gchar *loc = strchr (original+i, ']');
-			if (loc) {
-				i = loc - original + 1;
-				continue;
-			}
-		}
-
-		/* Remove {anything} */
-		if (original[i] == '{') {
-			gchar *loc = strchr (original+i, '}');
-			if (loc) {
-				i = loc - original + 1;
-				continue;
-			}
-		}
-
-		/* Remove <anything> */
-		if (original[i] == '<') {
-			gchar *loc = strchr (original+i, '>');
-			if (loc) {
-				i = loc - original + 1;
-				continue;
-			}
-		}
-
-		/* Remove double whitespaces */
-		if ((y > 0) &&
-		    (original[i] == ' ' || original[i] == '\t') &&
-		    (retval[y-1] == ' ' || retval[y-1] == '\t')) {
-			i++;
-			continue;
-		}
-
-		/* Remove strange characters */
-		if (!strchr (foo, original[i])) {
-			retval[y] = original[i]!='\t'?original[i]:' ';
-			y++;
-		}
-
-		i++;
+	if (open_pos) {
+		*open_pos = -1;
 	}
 
-	retval[y] = 0;
+	if (close_pos) {
+		*close_pos = -1;
+	}
 
-	return retval;
+	p1 = g_utf8_strchr (original, -1, open_char);
+	if (p1) {
+		if (open_pos) {
+			*open_pos = p1 - original;
+		}
+
+		p2 = g_utf8_strchr (g_utf8_next_char (p1), -1, close_char);
+		if (p2) {
+			if (close_pos) {
+				*close_pos = p2 - original;
+			}
+			
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+gchar *
+tracker_albumart_strip_invalid_entities (const gchar *original)
+{
+	GString         *str_no_blocks;
+	gchar          **strv;
+	gchar           *str;
+	gboolean         blocks_done = FALSE;
+	const gchar     *p;
+	const gchar     *invalid_chars = "()[]<>{}_!@#$^&*+=|\\/\"'?~";
+	const gchar     *invalid_chars_delimiter = "*";
+	const gchar     *convert_chars = "\t";
+	const gchar     *convert_chars_delimiter = " ";
+	const gunichar   blocks[5][2] = {
+		{ '(', ')' },
+		{ '{', '}' }, 
+		{ '[', ']' }, 
+		{ '<', '>' }, 
+		{  0,   0  }
+	};
+
+	str_no_blocks = g_string_new ("");
+
+	p = original;
+
+	while (!blocks_done) {
+		gint pos1, pos2, i;
+
+		pos1 = -1;
+		pos2 = -1;
+	
+		for (i = 0; blocks[i][0] != 0; i++) {
+			gint start, end;
+			
+			/* Go through blocks, find the earliest block we can */
+			if (strip_find_next_block (p, blocks[i][0], blocks[i][1], &start, &end)) {
+				if (pos1 == -1 || start < pos1) {
+					pos1 = start;
+					pos2 = end;
+				}
+			}
+		}
+		
+		/* If either are -1 we didn't find any */
+		if (pos1 == -1) {
+			/* This means no blocks were found */
+			g_string_append (str_no_blocks, p);
+			blocks_done = TRUE;
+		} else {
+			/* Append the test BEFORE the block */
+                        if (pos1 > 0) {
+                                g_string_append_len (str_no_blocks, p, pos1);
+                        }
+
+                        p = g_utf8_next_char (p + pos2);
+
+			/* Do same again for position AFTER block */
+			if (*p == '\0') {
+				blocks_done = TRUE;
+			}
+		}	
+	}
+
+	/* Now convert chars to lower case */
+	str = g_utf8_strdown (str_no_blocks->str, -1);
+	g_string_free (str_no_blocks, TRUE);
+
+	/* Now strip invalid chars */
+	g_strdelimit (str, invalid_chars, *invalid_chars_delimiter);
+	strv = g_strsplit (str, invalid_chars_delimiter, -1);
+	g_free (str);
+        str = g_strjoinv (NULL, strv);
+	g_strfreev (strv);
+
+	/* Now convert chars */
+	g_strdelimit (str, convert_chars, *convert_chars_delimiter);
+	strv = g_strsplit (str, convert_chars_delimiter, -1);
+	g_free (str);
+        str = g_strjoinv (convert_chars_delimiter, strv);
+	g_strfreev (strv);
+
+        /* Now remove double spaces */
+	strv = g_strsplit (str, "  ", -1);
+	g_free (str);
+        str = g_strjoinv (" ", strv);
+	g_strfreev (strv);
+        
+        /* Now strip leading/trailing white space */
+        g_strstrip (str);
+
+	return str;
 }
 
 void
@@ -317,11 +378,11 @@ tracker_albumart_heuristic (const gchar *artist_,
 			    const gchar *local_uri,
 			    gboolean    *copied)
 {
-	GFile *file;
+	GFile *file, *dirf;
 	GDir *dir;
 	struct stat st;
 	gchar *target = NULL;
-	gchar *basename;
+	gchar *dirname;
 	const gchar *name;
 	gboolean retval;
 	gint tracks;
@@ -339,13 +400,14 @@ tracker_albumart_heuristic (const gchar *artist_,
 			tracker_albumart_get_path (artist, album, 
 						   "album", NULL, 
 						   &target, NULL);
-			
-			file = g_file_new_for_path (target);
-			
-			g_file_copy_async (local_file, file, 0, 0, 
-					   NULL, NULL, NULL, NULL, NULL);
-			
-			g_object_unref (file);
+			if (target) {
+				file = g_file_new_for_path (target);
+				
+				g_file_copy_async (local_file, file, 0, 0, 
+						   NULL, NULL, NULL, NULL, NULL);
+				
+				g_object_unref (file);
+			}
 			g_object_unref (local_file);
 			
 			*copied = TRUE;
@@ -360,24 +422,32 @@ tracker_albumart_heuristic (const gchar *artist_,
 	*copied = FALSE;
 
 	file = g_file_new_for_path (filename);
-	basename = g_file_get_basename (file);
+	dirf = g_file_get_parent (file);
+	dirname = g_file_get_path (dirf);
 	g_object_unref (file);
+	g_object_unref (dirf);
 
-	if (!basename) {
+	if (!dirname) {
 		return FALSE;
 	}
 
-	dir = g_dir_open (basename, 0, NULL);
+	dir = g_dir_open (dirname, 0, NULL);
 
 	if (!dir) {
-		g_free (basename);
+		g_free (dirname);
 		return FALSE;
 	}
 
 	retval = FALSE;
 	file = NULL;
 
-	g_stat (basename, &st);
+	if (g_stat (dirname, &st) == -1) {
+		g_warning ("Could not g_stat() directory:'%s' for albumart heuristic",
+			   dirname);
+		g_free (dirname);
+		return FALSE;
+	}
+
 	count = st.st_nlink;
 	
 	if (tracks_str) {
@@ -387,17 +457,17 @@ tracker_albumart_heuristic (const gchar *artist_,
 	}
 
 	if (artist_) {
-		artist = strip_characters (artist_);
+		artist = tracker_albumart_strip_invalid_entities (artist_);
 	}
 
 	if (album_) {
-		album = strip_characters (album_);
+		album = tracker_albumart_strip_invalid_entities (album_);
 	}
 
 	/* If amount of files and amount of tracks in the album somewhat match */
 
 	if ((tracks != -1 && tracks < count + 3 && tracks > count - 3) || 
-	    (tracks == -1 && count > 8 && count < 50)) {
+	    (tracks == -1 && count >= 2 && count < 50)) {
 		gchar *found = NULL;
 
 		/* Try to find cover art in the directory */
@@ -409,63 +479,68 @@ tracker_albumart_heuristic (const gchar *artist_,
 				
 				if (g_str_has_suffix (name, "jpeg") || 
 				    g_str_has_suffix (name, "jpg")) {
-					GFile *file_found;
-					
 					if (!target) {
 						tracker_albumart_get_path (artist, album, 
 									   "album", NULL, 
 									   &target, NULL);
 					}
 					
-					if (!file) {
+					if (!file && target) {
 						file = g_file_new_for_path (target);
 					}
-					
-					found = g_build_filename (basename, name, NULL);
-					file_found = g_file_new_for_path (found);
-					
-					g_file_copy (file_found, file, 0, NULL, NULL, NULL, &error);
-					
-					if (!error) {
-						retval = TRUE;
-					} else {
-						g_error_free (error);
-						retval = FALSE;
-					}
-					
-					g_free (found);
-					g_object_unref (file_found);
-				} else {
-#ifdef HAVE_GDKPIXBUF
-					GdkPixbuf *pixbuf;
-					
-					found = g_build_filename (basename, name, NULL);
-					pixbuf = gdk_pixbuf_new_from_file (found, &error);
-					
-					if (error) {
-						g_error_free (error);
-						retval = FALSE;
-					} else {
-						if (!target) {
-							tracker_albumart_get_path (artist, 
-										   album, 
-										   "album", 
-										   NULL, 
-										   &target, 
-										   NULL);
-						}
-						
-						gdk_pixbuf_save (pixbuf, target, "jpeg", &error, NULL);
-						
+
+					if (file) {
+						GFile *file_found;
+
+						found = g_build_filename (dirname, name, NULL);
+						file_found = g_file_new_for_path (found);
+						g_file_copy (file_found, file, 0, NULL, NULL, NULL, &error);
+
 						if (!error) {
 							retval = TRUE;
 						} else {
 							g_error_free (error);
+							error = NULL;
 							retval = FALSE;
 						}
+						
+						g_free (found);
+						g_object_unref (file_found);
+					}
+				} else {
+#ifdef HAVE_GDKPIXBUF
+					if (g_str_has_suffix (name, "png")) {
+						GdkPixbuf *pixbuf;
+						
+						found = g_build_filename (dirname, name, NULL);
+						pixbuf = gdk_pixbuf_new_from_file (found, &error);
+						
+						if (error) {
+							g_error_free (error);
+							error = NULL;
+							retval = FALSE;
+						} else {
+							if (!target) {
+								tracker_albumart_get_path (artist, 
+											   album, 
+											   "album", 
+											   NULL, 
+											   &target, 
+											   NULL);
+							}
+							
+							gdk_pixbuf_save (pixbuf, target, "jpeg", &error, NULL);
+							
+							if (!error) {
+								retval = TRUE;
+							} else {
+								g_error_free (error);
+								retval = FALSE;
+							}
+						}
+						
+						g_free (found);
 					}
-					
-					g_free (found);
 #else  /* HAVE_GDKPIXBUF */
 					retval = FALSE;
 #endif /* HAVE_GDKPIXBUF */
@@ -486,7 +561,7 @@ tracker_albumart_heuristic (const gchar *artist_,
 	}
 
 	g_free (target);
-	g_free (basename);
+	g_free (dirname);
 	g_free (artist);
 	g_free (album);
 
@@ -594,13 +669,13 @@ tracker_albumart_get_path (const gchar  *a,
 	if (!a) {
 		f_a = g_strdup (" ");
 	} else {
-		f_a = strip_characters (a);
+		f_a = tracker_albumart_strip_invalid_entities (a);
 	}
 
 	if (!b) {
 		f_b = g_strdup (" ");
 	} else {
-		f_b = strip_characters (b); 
+		f_b = tracker_albumart_strip_invalid_entities (b); 
 	}
 
 	down1 = g_utf8_strdown (f_a, -1);
@@ -632,7 +707,7 @@ tracker_albumart_get_path (const gchar  *a,
 		gchar *local_dir;
 		GFile *file, *parent;
 
-		if (strchr (uri, ':')) {
+		if (strstr (uri, "://")) {
 			file = g_file_new_for_uri (uri);
 		} else {
 			file = g_file_new_for_path (uri); 
