@@ -29,13 +29,18 @@ struct TrackerDBusRequestHandler {
 	gpointer	       user_data;
 };
 
-static GSList *hooks;
+static GSList   *hooks;
+static gboolean  block_hooks;
 
 static void
 request_handler_call_for_new (guint request_id)
 {
 	GSList *l;
 
+	if (block_hooks) {
+		return;
+	}
+
 	for (l = hooks; l; l = l->next) {
 		TrackerDBusRequestHandler *handler;
 
@@ -52,6 +57,10 @@ request_handler_call_for_done (guint request_id)
 {
 	GSList *l;
 
+	if (block_hooks) {
+		return;
+	}
+
 	for (l = hooks; l; l = l->next) {
 		TrackerDBusRequestHandler *handler;
 
@@ -360,3 +369,15 @@ tracker_dbus_request_debug (gint	 request_id,
 		 str);
 	g_free (str);
 }
+
+void
+tracker_dbus_request_block_hooks (void)
+{
+	block_hooks = TRUE;
+}
+
+void
+tracker_dbus_request_unblock_hooks (void)
+{
+	block_hooks = FALSE;
+}
