@@ -1224,6 +1224,299 @@ tracker_data_search_get_count (const gchar	   *service_type,
 	return count;
 }
 
+TrackerDBResultSet *
+tracker_data_search_get_unique_values_with_aggregates (const gchar	      *service_type,
+						       gchar		     **fields,
+						       const gchar	      *query_condition,
+						       gchar                 **aggregates,
+						       gchar	             **aggregate_fields,
+						       gboolean	               order_desc,
+						       gint		       offset,
+						       gint		       max_hits,
+						       GError	             **error)
+{
+	TrackerDBInterface *iface;
+	TrackerDBResultSet *result_set = NULL;
+
+	GSList		   *field_list = NULL;
+	gchar		   *str_offset, *str_limit;
+
+	GString		   *sql_select;
+	GString		   *sql_from;
+	GString		   *sql_where;
+	GString		   *sql_order;
+	GString		   *sql_group;
+	gchar		   *sql;
+
+	gchar		   *rdf_where;
+	gchar		   *rdf_from;
+	GError		   *actual_error = NULL;
+
+	guint		    i;
+
+	g_return_val_if_fail (service_type != NULL, NULL);
+	g_return_val_if_fail (fields != NULL, NULL);
+	g_return_val_if_fail (query_condition != NULL, NULL);
+
+	if (!tracker_ontology_service_is_valid (service_type)) {
+		g_set_error (error, TRACKER_DBUS_ERROR, 0,
+			     "Service_Type '%s' is invalid or has not been implemented yet",
+			     service_type);
+		return NULL;
+	}
+
+	if (g_strv_length (aggregates) != g_strv_length (aggregate_fields)) {
+		g_set_error (error, TRACKER_DBUS_ERROR, 0,
+			     "The number of aggregates and aggregate fields do not match");
+		return NULL;	
+	}
+
+	iface = tracker_db_manager_get_db_interface_by_service (service_type);
+
+	sql_select = g_string_new ("SELECT DISTINCT ");
+	sql_from   = g_string_new ("\nFROM Services AS S ");
+	sql_where  = g_string_new ("\nWHERE ");
+	sql_order  = g_string_new ("");
+	sql_group  = g_string_new ("\nGROUP BY ");
+
+
+	for (i = 0; i < g_strv_length (fields); i++) {
+		TrackerFieldData *fd;
+
+		fd = tracker_metadata_add_metadata_field (iface, service_type, &field_list, fields[i], TRUE, FALSE, TRUE);
+
+		if (!fd) {
+			g_string_free (sql_select, TRUE);
+			g_string_free (sql_from, TRUE);
+			g_string_free (sql_where, TRUE);
+			g_string_free (sql_order, TRUE);
+			g_string_free (sql_group, TRUE);
+
+			g_slist_foreach (field_list, (GFunc) g_object_unref, NULL);
+			g_slist_free (field_list);
+
+			g_set_error (error, TRACKER_DBUS_ERROR, 0,
+				     "Invalid or non-existant metadata type '%s' specified",
+				     fields[i]);
+			return NULL;
+		}
+
+		if (i) {
+			g_string_append_printf (sql_select, ",");
+			g_string_append_printf (sql_group, ",");
+		}
+
+		g_string_append_printf (sql_select, "%s", tracker_field_data_get_select_field (fd));
+		if (order_desc) {
+			if (i) {
+				g_string_append_printf (sql_order, ",");
+			}
+			g_string_append_printf (sql_order, "\nORDER BY %s DESC ",
+						tracker_field_data_get_order_field (fd));
+		}
+		g_string_append_printf (sql_group, "%s", tracker_field_data_get_order_field (fd));
+
+	}
+
+	for (i = 0; i < g_strv_length (aggregates); i++) {
+		if (strcmp (aggregates[i],"COUNT") == 0) {
+			if (!(tracker_is_empty_string (aggregate_fields[i]))) {
+				TrackerFieldData *fd;
+				
+				if (strcmp (aggregate_fields[i], "*")) {
+					fd = tracker_metadata_add_metadata_field (iface, service_type,
+										  &field_list, aggregate_fields[i],
+										  TRUE, FALSE, FALSE);
+					
+					if (!fd) {
+						g_string_free (sql_select, TRUE);
+						g_string_free (sql_from, TRUE);
+						g_string_free (sql_where, TRUE);
+						g_string_free (sql_order, TRUE);
+						g_string_free (sql_group, TRUE);
+
+						g_slist_foreach (field_list, (GFunc) g_object_unref, NULL);
+						g_slist_free (field_list);
+						
+						g_set_error (error, TRACKER_DBUS_ERROR, 0,
+							     "Invalid or non-existant metadata type '%s' specified",
+							     aggregate_fields[i]);
+						return NULL;
+					}
+					
+					g_string_append_printf (sql_select, ", COUNT (DISTINCT %s)",
+								tracker_field_data_get_select_field (fd));
+				} else {
+					g_string_append_printf (sql_select, ", COUNT (DISTINCT S.ID)");		
+				}
+			}
+		} else if (strcmp (aggregates[i], "SUM") == 0) {
+			if ( !(tracker_is_empty_string (aggregate_fields[i])) ) {
+				TrackerFieldData *fd;
+				TrackerFieldType  data_type;
+				
+				fd = tracker_metadata_add_metadata_field (iface, service_type, 
+									  &field_list, aggregate_fields[i],
+									  TRUE, FALSE, FALSE);
+				
+				if (!fd) {
+					g_string_free (sql_select, TRUE);
+					g_string_free (sql_from, TRUE);
+					g_string_free (sql_where, TRUE);
+					g_string_free (sql_order, TRUE);
+					g_string_free (sql_group, TRUE);
+					
+					g_slist_foreach (field_list, (GFunc) g_object_unref, NULL);
+					g_slist_free (field_list);
+
+					g_set_error (error, TRACKER_DBUS_ERROR, 0,
+						     "Invalid or non-existant metadata type '%s' specified",
+						     aggregate_fields[i]);
+					return NULL;
+				}
+				
+				data_type = tracker_field_data_get_data_type (fd);
+				
+				if (!is_data_type_numeric (data_type)) {
+					g_string_free (sql_select, TRUE);
+					g_string_free (sql_from, TRUE);
+					g_string_free (sql_where, TRUE);
+					g_string_free (sql_order, TRUE);
+					g_string_free (sql_group, TRUE);
+					
+					g_slist_foreach (field_list, (GFunc) g_object_unref, NULL);
+					g_slist_free (field_list);
+
+					g_set_error (error, TRACKER_DBUS_ERROR, 0,
+						     "Cannot sum '%s': this metadata type is not numeric",
+						     aggregate_fields[i]);
+					return NULL;
+				}
+				
+				g_string_append_printf (sql_select, ", SUM (%s)",
+							tracker_field_data_get_select_field (fd));
+			}
+		} else if (strcmp (aggregates[i], "CONCAT") == 0 ) { 
+			if (!(tracker_is_empty_string (aggregate_fields[i]))) {
+				TrackerFieldData *fd;
+				TrackerFieldType  data_type;
+				
+				fd = tracker_metadata_add_metadata_field (iface, service_type,
+									  &field_list, aggregate_fields[i],
+									  TRUE, FALSE, FALSE);
+				
+				if (!fd) {
+					g_string_free (sql_select, TRUE);
+					g_string_free (sql_from, TRUE);
+					g_string_free (sql_where, TRUE);
+					g_string_free (sql_order, TRUE);
+					g_string_free (sql_group, TRUE);
+					
+					g_slist_foreach (field_list, (GFunc) g_object_unref, NULL);
+					g_slist_free (field_list);
+
+					g_set_error (error, TRACKER_DBUS_ERROR, 0,
+						     "Invalid or non-existant metadata type '%s' specified",
+						     aggregate_fields[i]);
+					return NULL;
+				}
+				
+				data_type = tracker_field_data_get_data_type (fd);
+				
+				if (!is_data_type_text (data_type)) {
+					g_string_free (sql_select, TRUE);
+					g_string_free (sql_from, TRUE);
+					g_string_free (sql_where, TRUE);
+					g_string_free (sql_order, TRUE);
+					g_string_free (sql_group, TRUE);
+					
+					g_slist_foreach (field_list, (GFunc) g_object_unref, NULL);
+					g_slist_free (field_list);
+
+					g_set_error (error, TRACKER_DBUS_ERROR, 0,
+						     "Cannot concatenate '%s': this metadata type is not text",
+						     aggregate_fields[i]);
+					return NULL;
+				}
+				
+				g_string_append_printf (sql_select, ", GROUP_CONCAT (DISTINCT %s)",
+							tracker_field_data_get_select_field (fd));
+			}
+		} else {
+			g_string_free (sql_select, TRUE);
+			g_string_free (sql_from, TRUE);
+			g_string_free (sql_where, TRUE);
+			g_string_free (sql_order, TRUE);
+			g_string_free (sql_group, TRUE);
+
+			g_slist_foreach (field_list, (GFunc) g_object_unref, NULL);
+			g_slist_free (field_list);
+
+			g_debug ("Got an unknown operation %s", aggregates[i]);
+			
+			g_set_error (error, TRACKER_DBUS_ERROR, 0,
+				     "Aggregate operation %s not found",
+				     aggregates[i]);
+			return NULL;
+		}
+	}
+	
+	tracker_rdf_filter_to_sql (iface, query_condition, service_type,
+				   &field_list, &rdf_from, &rdf_where, &actual_error);
+
+	if (actual_error) {
+		g_string_free (sql_select, TRUE);
+		g_string_free (sql_from, TRUE);
+		g_string_free (sql_where, TRUE);
+		g_string_free (sql_order, TRUE);
+		g_string_free (sql_group, TRUE);
+
+		g_slist_foreach (field_list, (GFunc) g_object_unref, NULL);
+		g_slist_free (field_list);
+
+		g_propagate_error (error, actual_error);
+
+		return NULL;
+	}
+
+	g_string_append_printf (sql_from, " %s ", rdf_from);
+	g_string_append_printf (sql_where, " %s ", rdf_where);
+
+	g_free (rdf_from);
+	g_free (rdf_where);
+
+	str_offset = tracker_gint_to_string (offset);
+	str_limit = tracker_gint_to_string (metadata_sanity_check_max_hits (max_hits));
+
+	g_string_append_printf (sql_order, " LIMIT %s,%s", str_offset, str_limit);
+
+	sql = g_strconcat (sql_select->str, " ",
+			   sql_from->str, " ",
+			   sql_where->str, " ",
+			   sql_group->str, " ",
+			   sql_order->str, NULL);
+
+	g_free (str_offset);
+	g_free (str_limit);
+
+	g_string_free (sql_select, TRUE);
+	g_string_free (sql_from, TRUE);
+	g_string_free (sql_where, TRUE);
+	g_string_free (sql_order, TRUE);
+	g_string_free (sql_group, TRUE);
+
+	g_slist_foreach (field_list, (GFunc) g_object_unref, NULL);
+	g_slist_free (field_list);
+
+	g_message ("Unique values query executed:\n%s", sql);
+
+	result_set =  tracker_db_interface_execute_query (iface, NULL, "%s", sql);
+
+	g_free (sql);
+
+	return result_set;
+}
+
 TrackerDBResultSet *
 tracker_data_search_metadata_in_path (const gchar	       *path,
 				      gchar		      **fields,
