@@ -35,10 +35,14 @@ typedef enum {
 	TRACKER_DB_CACHE,
 	TRACKER_DB_FILE_METADATA,
 	TRACKER_DB_FILE_CONTENTS,
+#ifdef HAVE_SQLITE_FTS
 	TRACKER_DB_FILE_FULLTEXT,
+#endif /* HAVE_SQLITE_FTS */
 	TRACKER_DB_EMAIL_METADATA,
 	TRACKER_DB_EMAIL_CONTENTS,
+#ifdef HAVE_SQLITE_FTS
 	TRACKER_DB_EMAIL_FULLTEXT,
+#endif /* HAVE_SQLITE_FTS */
 } TrackerDB;
 
 typedef enum {
