@@ -56,7 +56,6 @@
 #include <libtracker-common/tracker-config.h>
 #include <libtracker-common/tracker-dbus.h>
 #include <libtracker-common/tracker-file-utils.h>
-#include <libtracker-common/tracker-hal.h>
 #include <libtracker-common/tracker-language.h>
 #include <libtracker-common/tracker-parser.h>
 #include <libtracker-common/tracker-ontology.h>
@@ -96,12 +95,12 @@
 #define TRACKER_INDEXER_ERROR_CODE  0
 
 /* Properties that change in move event */
-#define METADATA_FILE_NAME_DELIMITED "File:NameDelimited"
-#define METADATA_FILE_EXT	     "File:Ext"
 #define METADATA_FILE_PATH	     "File:Path"
 #define METADATA_FILE_NAME	     "File:Name"
 #define METADATA_FILE_MIMETYPE       "File:Mime"
 
+#undef ENABLE_TTL_LOADER
+
 typedef struct PathInfo PathInfo;
 typedef struct MetadataForeachData MetadataForeachData;
 typedef struct MetadataRequest MetadataRequest;
@@ -166,9 +165,12 @@ struct PathInfo {
 };
 
 struct MetadataForeachData {
+	TrackerDataUpdateMetadataContext *context;
 	TrackerLanguage *language;
 	TrackerConfig *config;
+	guint32 service_id;
 	TrackerService *service;
+	TrackerDBIndex *index;
 	gboolean add;
 	guint32 id;
 };
@@ -180,9 +182,10 @@ struct UpdateWordsForeachData {
 
 enum TrackerIndexerState {
 	TRACKER_INDEXER_STATE_INDEX_OVERLOADED = 1 << 0,
-	TRACKER_INDEXER_STATE_PAUSED	= 1 << 1,
-	TRACKER_INDEXER_STATE_STOPPED	= 1 << 2,
-	TRACKER_INDEXER_STATE_CLEANUP   = 1 << 3
+	TRACKER_INDEXER_STATE_INDEX_ERROR = 1 << 1,
+	TRACKER_INDEXER_STATE_PAUSED	= 1 << 2,
+	TRACKER_INDEXER_STATE_STOPPED	= 1 << 3,
+	TRACKER_INDEXER_STATE_CLEANUP   = 1 << 4
 };
 
 enum {
@@ -214,6 +217,10 @@ static void	state_check	       (TrackerIndexer	    *indexer);
 static void     check_finished         (TrackerIndexer      *indexer,
 					gboolean             interrupted);
 
+static void     item_erase             (TrackerIndexer      *indexer,
+					TrackerService      *service,
+					guint32              service_id);
+
 static gboolean item_process           (TrackerIndexer      *indexer,
 					PathInfo            *info,
 					const gchar         *dirname,
@@ -432,7 +439,6 @@ tracker_indexer_transaction_commit (TrackerIndexer *indexer)
 {
 	stop_transaction (indexer);
 	tracker_indexer_set_running (indexer, TRUE);
-
 }
 
 void
@@ -442,6 +448,14 @@ tracker_indexer_transaction_open (TrackerIndexer *indexer)
 	start_transaction (indexer);
 }
 
+TrackerHal *
+tracker_indexer_get_hal (TrackerIndexer *indexer)
+{
+	g_return_val_if_fail (TRACKER_IS_INDEXER (indexer), NULL);
+
+	return indexer->private->hal;
+}
+
 #ifdef HAVE_HAL
 
 static void
@@ -535,6 +549,7 @@ index_error_received_cb (TrackerDBIndex *index,
 			 const GError   *error,
 			 TrackerIndexer *indexer)
 {
+	state_set_flags (indexer, TRACKER_INDEXER_STATE_INDEX_ERROR);
 	g_signal_emit (indexer, signals[INDEXING_ERROR], 0,
 		       error->message, TRUE);
 }
@@ -691,10 +706,6 @@ tracker_indexer_get_property (GObject	 *object,
 			      GValue	 *value,
 			      GParamSpec *pspec)
 {
-	TrackerIndexerPrivate *priv;
-
-	priv = TRACKER_INDEXER_GET_PRIVATE (object);
-
 	switch (prop_id) {
 	case PROP_RUNNING:
 		g_value_set_boolean (value,
@@ -1123,13 +1134,12 @@ index_metadata_item (TrackerField	 *field,
 		     const gchar	 *value,
 		     MetadataForeachData *data)
 {
-	TrackerDBIndex *lindex;
+	gchar *word, *end;
 	gchar *parsed_value;
-	gchar **arr;
-	gint service_id;
-	gint i;
 	gint score;
+	gboolean reached_end = FALSE;
 
+#ifdef ENABLE_FTS
 	parsed_value = tracker_parser_text_to_string (value,
 						      data->language,
 						      tracker_config_get_max_word_length (data->config),
@@ -1137,7 +1147,25 @@ index_metadata_item (TrackerField	 *field,
 						      tracker_field_get_filtered (field),
 						      tracker_field_get_filtered (field),
 						      tracker_field_get_delimited (field));
+#else
+	parsed_value = NULL;
+#endif /* ENABLE_FTS */
 
+	if (data->add) {
+		tracker_data_update_set_metadata (data->context,
+						  data->service,
+						  data->id,
+						  field,
+						  value,
+						  parsed_value);
+	} else {
+		tracker_data_update_delete_metadata (data->service,
+						     data->id,
+						     field,
+						     value);
+	}
+
+#ifdef ENABLE_FTS
 	if (!parsed_value) {
 		return;
 	}
@@ -1148,26 +1176,48 @@ index_metadata_item (TrackerField	 *field,
 		score = -1 * tracker_field_get_weight (field);
 	}
 
-	arr = g_strsplit (parsed_value, " ", -1);
-	service_id = tracker_service_get_id (data->service);
-	lindex = tracker_db_index_manager_get_index_by_service_id (service_id);
+	word = parsed_value;
 
-	for (i = 0; arr[i]; i++) {
-		tracker_db_index_add_word (lindex,
-					   arr[i],
+	while (word && *word) {
+		/* Skip whitespaces */
+		while (*word == ' ') {
+			word++;
+		}
+
+		if (!*word) {
+			/* Reached end of the string */
+			break;
+		}
+
+		/* Find end of word */
+		end = word;
+
+		while (*end != ' ' && *end != '\0') {
+			end++;
+		}
+
+		if (!*end) {
+			/* Reached end of the string */
+			reached_end = TRUE;
+		}
+
+		*end = '\0';
+
+		tracker_db_index_add_word (data->index,
+					   word,
 					   data->id,
-					   tracker_service_get_id (data->service),
+					   data->service_id,
 					   score);
+
+		if (!reached_end) {
+			word = end + 1;
+		} else {
+			word = NULL;
+		}
 	}
 
-	if (data->add) {
-		tracker_data_update_set_metadata (data->service, data->id, field, (gchar *) value, parsed_value);
-	} else {
-		tracker_data_update_delete_metadata (data->service, data->id, field, (gchar *)value);
-	}
-
-	g_strfreev (arr);
 	g_free (parsed_value);
+#endif
 }
 
 static void
@@ -1206,15 +1256,22 @@ index_metadata_foreach (TrackerField *field,
 
 static void
 index_metadata (TrackerIndexer	      *indexer,
+		TrackerDataUpdateMetadataContext *context,
 		guint32		       id,
 		TrackerService	      *service,
 		TrackerModuleMetadata *metadata)
 {
 	MetadataForeachData data;
+	gint service_id;
 
+	service_id = tracker_service_get_id (service);
+
+	data.context = context;
 	data.language = indexer->private->language;
 	data.config = indexer->private->config;
+	data.service_id = service_id;
 	data.service = service;
+	data.index = tracker_db_index_manager_get_index_by_service_id (service_id);
 	data.id = id;
 	data.add = TRUE;
 
@@ -1230,10 +1287,16 @@ unindex_metadata (TrackerIndexer      *indexer,
 		  TrackerDataMetadata *metadata)
 {
 	MetadataForeachData data;
+	gint service_id;
 
+	service_id = tracker_service_get_id (service);
+
+	data.context = NULL;
 	data.language = indexer->private->language;
 	data.config = indexer->private->config;
+	data.service_id = service_id;
 	data.service = service;
+	data.index = tracker_db_index_manager_get_index_by_service_id (service_id);
 	data.id = id;
 	data.add = FALSE;
 
@@ -1567,22 +1630,26 @@ generate_item_thumbnail (TrackerIndexer        *indexer,
 			 const gchar           *basename,
 			 TrackerModuleMetadata *metadata)
 {
-	const gchar *path, *mime_type;
+	const gchar *mime_type;
 
-	path = tracker_module_metadata_lookup (metadata, METADATA_FILE_NAME_DELIMITED, FALSE);
 	mime_type = tracker_module_metadata_lookup (metadata, METADATA_FILE_MIMETYPE, FALSE);
 
-	if (path && path[0] == G_DIR_SEPARATOR && mime_type &&
+	if (dirname && 
+	    dirname[0] == G_DIR_SEPARATOR && 
+	    mime_type &&
 	    tracker_config_get_enable_thumbnails (indexer->private->config)) {
 		GFile *file;
 		gchar *uri;
+		gchar *path;
 
+		path = g_build_filename (dirname, basename, NULL);
 		file = g_file_new_for_path (path);
+		g_free (path);
+
 		uri = g_file_get_uri (file);
+		g_object_unref (file);
 
 		tracker_thumbnailer_queue_file (uri, mime_type);
-
-		g_object_unref (file);
 		g_free (uri);
 	}
 }
@@ -1599,6 +1666,7 @@ item_add_or_update (TrackerIndexer        *indexer,
 	guint32 id;
 	gchar *mount_point = NULL;
 	gchar *service_path;
+	gboolean exists, enabled;
 
 	service = get_service_for_file (info->module_file, info->module);
 
@@ -1606,22 +1674,37 @@ item_add_or_update (TrackerIndexer        *indexer,
 		return;
 	}
 
-	if (tracker_data_query_service_exists (service, dirname, basename, &id, NULL)) {
+	if (G_UNLIKELY (!indexer->private->in_transaction)) {
+		start_transaction (indexer);
+	}
+
+	service_path = g_build_path (G_DIR_SEPARATOR_S,
+				     dirname,
+				     basename,
+				     NULL);
+
+	exists = tracker_data_query_service_exists (service, dirname, basename, &id, NULL, &enabled);
+
+	if (exists && !enabled) {
+		/* Service must be deleted before insertion */
+		item_erase (indexer, service, id);
+		exists = FALSE;
+	}
+
+	if (exists) {
+		TrackerDataUpdateMetadataContext *context;
 		TrackerDataMetadata *old_metadata_emb, *old_metadata_non_emb;
 		gchar *old_text;
 
 		if (tracker_module_file_get_flags (info->module_file) & TRACKER_FILE_CONTENTS_STATIC) {
 			/* According to the module, the metadata can't change for this item */
-			g_debug ("Not updating static item '%s/%s'",
-				 dirname,
-				 basename);
+			g_debug ("Not updating static item '%s'", service_path);
+			g_free (service_path);
 			return;
 		}
 
 		/* Update case */
-		g_debug ("Updating item '%s/%s'", 
-			 dirname, 
-			 basename);
+		g_debug ("Updating item '%s'", service_path);
 
 		/* "metadata" (new metadata) contains embedded props and can contain
 		 * non-embedded properties with default values! Dont overwrite those 
@@ -1641,7 +1724,13 @@ item_add_or_update (TrackerIndexer        *indexer,
 							remove_existing_non_emb_metadata,
 							old_metadata_non_emb);
 
-		index_metadata (indexer, id, service, metadata);
+		context = tracker_data_update_metadata_context_new (TRACKER_CONTEXT_TYPE_UPDATE,
+								    service, id);
+
+		index_metadata (indexer, context, id, service, metadata);
+
+		tracker_data_update_metadata_context_close (context);
+		tracker_data_update_metadata_context_free (context);
 
 		/* Take the old text -> the new one, calculate
 		 * difference and add the words.
@@ -1651,39 +1740,43 @@ item_add_or_update (TrackerIndexer        *indexer,
 		item_update_content (indexer, service, id, old_text, text);
 
 		if (strcmp (tracker_service_get_name (service), "Folders") == 0) {
-			gchar *path;
-
 			/* Remove no longer existing children, this is necessary in case
 			 * there were files added/removed in a directory between tracker
 			 * executions
 			 */
-
-			path = g_build_path (G_DIR_SEPARATOR_S, dirname, basename, NULL);
-			remove_stale_children (indexer, service, info, path);
-			g_free (path);
+			remove_stale_children (indexer, service, info, service_path);
 		}
 
 		g_free (old_text);
 		tracker_data_metadata_free (old_metadata_emb);
 		tracker_data_metadata_free (old_metadata_non_emb);
 	} else {
+		TrackerDataUpdateMetadataContext *context;
 		GHashTable *data;
+		const gchar *udi;
 
-		g_debug ("Adding item '%s/%s'",
-			 dirname,
-			 basename);
+		g_debug ("Adding item '%s'", service_path);
 
 		/* Service wasn't previously indexed */
 		id = tracker_data_update_get_new_service_id (indexer->private->common);
 		data = tracker_module_metadata_get_hash_table (metadata);
+		udi = tracker_hal_udi_get_for_path (indexer->private->hal, service_path);
 
-		tracker_data_update_create_service (service,
+		context = tracker_data_update_metadata_context_new (TRACKER_CONTEXT_TYPE_INSERT,
+								    service, id);
+
+		tracker_data_update_create_service (context,
+						    service,
 						    id,
+						    udi,
 						    dirname,
 						    basename,
 						    data);
 
-		index_metadata (indexer, id, service, metadata);
+		index_metadata (indexer, context, id, service, metadata);
+
+		tracker_data_update_metadata_context_close (context);
+		tracker_data_update_metadata_context_free (context);
 
 		if (text) {
 			/* Save in the index */
@@ -1702,26 +1795,21 @@ item_add_or_update (TrackerIndexer        *indexer,
 
 	generate_item_thumbnail (indexer, dirname, basename, metadata);
 
-	/* TODO: URI branch path -> uri */
-
-	service_path = g_build_path (G_DIR_SEPARATOR_S, 
-				     dirname, 
-				     basename, 
-				     NULL);
-
+#ifdef ENABLE_TTL_LOADER
 #ifdef HAVE_HAL
 	if (tracker_hal_path_is_on_removable_device (indexer->private->hal,
 						     service_path, 
 						     &mount_point,
 						     NULL)) {
-
 		tracker_removable_device_add_metadata (indexer, 
 						       mount_point, 
 						       service_path, 
 						       tracker_service_get_name (service),
 						       metadata);
 	}
-#endif
+#endif /* HAVE_HAL */
+#endif /* ENABLE_TTL_LOADER */
+
 	g_free (mount_point);
 	g_free (service_path);
 }
@@ -1736,13 +1824,12 @@ filter_invalid_after_move_properties (TrackerField *field,
 
 	name = tracker_field_get_name (field);
 
-	if (g_strcmp0 (name, METADATA_FILE_NAME_DELIMITED) == 0 ||
-	    g_strcmp0 (name, METADATA_FILE_NAME) == 0 ||
-	    g_strcmp0 (name, METADATA_FILE_PATH) == 0 ||
-	    g_strcmp0 (name, METADATA_FILE_EXT) == 0) {
+	if (g_strcmp0 (name, METADATA_FILE_NAME) == 0 ||
+	    g_strcmp0 (name, METADATA_FILE_PATH) == 0) {
 		return FALSE;
 	}
 
+
 	return TRUE;
 }
 
@@ -1782,6 +1869,8 @@ update_moved_item_removable_device (TrackerIndexer *indexer,
 				    GFile          *file,
 				    GFile          *source_file)
 {
+#ifdef ENABLE_TTL_LOADER
+#ifdef HAVE_HAL
 	const gchar *service_name;
 	gchar *path, *source_path;
 	gchar *mount_point = NULL;
@@ -1790,23 +1879,19 @@ update_moved_item_removable_device (TrackerIndexer *indexer,
 	path = g_file_get_path (file);
 	source_path = g_file_get_path (source_file);
 
-#ifdef HAVE_HAL
 	if (tracker_hal_path_is_on_removable_device (indexer->private->hal,
 						     source_path,
 						     &mount_point,
-						     NULL) ) {
-
+						     NULL)) {
 		if (tracker_hal_path_is_on_removable_device (indexer->private->hal,
-						     path,
-						     NULL,
-						     NULL) ) {
-
+							     path,
+							     NULL,
+							     NULL)) {
 			tracker_removable_device_add_move (indexer,
 							   mount_point,
 							   source_path,
 							   path,
 							   service_name);
-
 		} else {
 			tracker_removable_device_add_removal (indexer,
 							      mount_point,
@@ -1814,10 +1899,12 @@ update_moved_item_removable_device (TrackerIndexer *indexer,
 							      service_name);
 		}
 	}
-#endif
+
 	g_free (mount_point);
 	g_free (source_path);
 	g_free (path);
+#endif /* HAVE_HAL */
+#endif /* ENABLE_TTL_LOADER */
 }
 
 static void
@@ -1828,9 +1915,9 @@ update_moved_item_index (TrackerIndexer      *indexer,
 			 GFile               *file,
 			 GFile               *source_file)
 {
+	TrackerDataUpdateMetadataContext *context;
 	TrackerModuleMetadata *new_metadata;
 	gchar *path, *new_path, *new_name;
-	const gchar *ext;
 
 	path = g_file_get_path (file);
 
@@ -1849,15 +1936,14 @@ update_moved_item_index (TrackerIndexer      *indexer,
 
 	tracker_module_metadata_add_string (new_metadata, METADATA_FILE_PATH, new_path);
 	tracker_module_metadata_add_string (new_metadata, METADATA_FILE_NAME, new_name);
-	tracker_module_metadata_add_string (new_metadata, METADATA_FILE_NAME_DELIMITED, path);
 
-	ext = strrchr (path, '.');
-	if (ext) {
-		ext++;
-		tracker_module_metadata_add_string (new_metadata, METADATA_FILE_EXT, ext);
-	}
+	context = tracker_data_update_metadata_context_new (TRACKER_CONTEXT_TYPE_UPDATE,
+							    service, service_id);
 
-	index_metadata (indexer, service_id, service, new_metadata);
+	index_metadata (indexer, context, service_id, service, new_metadata);
+
+	tracker_data_update_metadata_context_close (context);
+	tracker_data_update_metadata_context_free (context);
 
 	g_object_unref (new_metadata);
 	g_free (new_path);
@@ -1885,7 +1971,9 @@ item_erase (TrackerIndexer *indexer,
 		gchar *uri;
 
 		/* TODO URI branch: this is a URI conversion */
-		path = tracker_data_metadata_lookup (data_metadata, "File:NameDelimited");
+		path = g_build_path (tracker_data_metadata_lookup (data_metadata, "File:Path"),
+				     tracker_data_metadata_lookup (data_metadata, "File:Name"),
+				     NULL);
 		file = g_file_new_for_path (path);
 		uri = g_file_get_uri (file);
 		g_object_unref (file);
@@ -1949,6 +2037,7 @@ item_move (TrackerIndexer  *indexer,
 	gchar *dest_dirname, *dest_basename;
 	guint32 service_id, dest_service_id;
 	GHashTable *children = NULL;
+	gboolean exists, enabled;
 
 	service = get_service_for_file (info->module_file, info->module);
 
@@ -1962,11 +2051,14 @@ item_move (TrackerIndexer  *indexer,
 	g_debug ("Moving item from '%s' to '%s'", source_path, path);
 
 	/* Get 'source' ID */
-	if (!tracker_data_query_service_exists (service,
-						dirname,
-						basename,
-						&service_id,
-						NULL)) {
+	exists = tracker_data_query_service_exists (service,
+						    dirname,
+						    basename,
+						    &service_id,
+						    NULL,
+						    &enabled);
+
+	if (!exists || !enabled) {
 		gchar *dest_dirname, *dest_basename;
 		gboolean res;
 
@@ -1990,7 +2082,7 @@ item_move (TrackerIndexer  *indexer,
 					       dest_dirname,
 					       dest_basename,
 					       &dest_service_id,
-					       NULL)) {
+					       NULL, NULL)) {
 		g_message ("Destination file '%s' already existed in database, removing", path);
 
 		/* Item has to be deleted from the database immediately */
@@ -2075,11 +2167,16 @@ item_mark_for_removal (TrackerIndexer *indexer,
 	const gchar *service_type;
 	guint service_id, service_type_id;
 	GHashTable *children = NULL;
+	gboolean exists, enabled;
 
 	g_debug ("Removing item: '%s/%s' (no metadata was given by module)", 
 		 dirname, 
 		 basename);
 
+	if (G_UNLIKELY (!indexer->private->in_transaction)) {
+		start_transaction (indexer);
+	}
+
 	/* The file is not anymore in the filesystem. Obtain
 	 * the service type from the DB.
 	 */
@@ -2093,9 +2190,9 @@ item_mark_for_removal (TrackerIndexer *indexer,
 	service_type = tracker_ontology_get_service_by_id (service_type_id);
 	service = tracker_ontology_get_service_by_name (service_type);
 
-	tracker_data_query_service_exists (service, dirname, basename, &service_id, NULL);
+	exists = tracker_data_query_service_exists (service, dirname, basename, &service_id, NULL, &enabled);
 
-	if (service_id < 1) {
+	if (!exists || !enabled) {
 		g_debug ("  File does not exist anyway "
 			 "(dirname:'%s', basename:'%s')",
 			 dirname, basename);
@@ -2134,17 +2231,20 @@ item_mark_for_removal (TrackerIndexer *indexer,
 
 		g_hash_table_destroy (children);
 	}
+
+#ifdef ENABLE_TTL_LOADER
 #ifdef HAVE_HAL
 	if (tracker_hal_path_is_on_removable_device (indexer->private->hal,
 						     path, 
 						     &mount_point,
 						     NULL)) {
-
 		tracker_removable_device_add_removal (indexer, mount_point, 
 						      path,
 						      tracker_service_get_name (service));
 	}
-#endif
+#endif /* HAVE_HAL */
+#endif /* ENABLE_TTL_LOADER */
+
 	g_free (mount_point);
 	g_free (path);
 }
@@ -2156,7 +2256,6 @@ item_process (TrackerIndexer *indexer,
 	      const gchar    *basename)
 {
 	TrackerModuleMetadata *metadata;
-	gchar *text;
 
 	metadata = tracker_module_file_get_metadata (info->module_file);
 
@@ -2169,7 +2268,13 @@ item_process (TrackerIndexer *indexer,
 	}
 
 	if (metadata) {
+		gchar *text;
+
+#ifdef ENABLE_FTS
 		text = tracker_module_file_get_text (info->module_file);
+#else  /* ENABLE_FTS */
+		text = NULL;
+#endif /* ENABLE_FTS */
 
 		if (tracker_module_file_is_cancelled (info->module_file)) {
 			g_object_unref (metadata);
@@ -2201,6 +2306,7 @@ handle_metadata_add (TrackerIndexer *indexer,
 		     GStrv	     values,
 		     GError	   **error)
 {
+	TrackerDataUpdateMetadataContext *context;
 	TrackerService *service;
 	TrackerField   *field;
 	guint           service_id, i, j;
@@ -2208,6 +2314,7 @@ handle_metadata_add (TrackerIndexer *indexer,
 	gchar          *joined, *dirname = NULL, *basename = NULL;
 	gchar         **old_contents;
 	gint            len;
+	gboolean        exists, enabled;
 
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
@@ -2247,15 +2354,16 @@ handle_metadata_add (TrackerIndexer *indexer,
 
 	tracker_file_get_path_and_name (uri, &dirname, &basename);
 
-	tracker_data_query_service_exists (service,
-				  dirname,
-				  basename,
-				  &service_id,
-				  NULL);
+	exists = tracker_data_query_service_exists (service,
+						    dirname,
+						    basename,
+						    &service_id,
+						    NULL,
+						    &enabled);
 	g_free (dirname);
 	g_free (basename);
 
-	if (service_id < 1) {
+	if (!exists || !enabled) {
 		g_set_error (error,
 			     g_quark_from_string (TRACKER_INDEXER_ERROR),
 			     TRACKER_INDEXER_ERROR_CODE,
@@ -2293,6 +2401,9 @@ handle_metadata_add (TrackerIndexer *indexer,
 
 	set_values = g_new0 (gchar *, g_strv_length (values) + 1);
 
+	context = tracker_data_update_metadata_context_new (TRACKER_CONTEXT_TYPE_UPDATE,
+							    service, service_id);
+
 	for (i = 0, j = 0; values[i] != NULL; i++) {
 		g_debug ("Setting metadata: service_type '%s' id '%d' field '%s' value '%s'",
 			 tracker_service_get_name (service),
@@ -2300,15 +2411,23 @@ handle_metadata_add (TrackerIndexer *indexer,
 			 tracker_field_get_name (field),
 			 values[i]);
 
-		if (tracker_field_get_multiple_values (field) 
-		    && (tracker_string_in_string_list (values[i], old_contents) > -1) ) {
+		if (tracker_field_get_multiple_values (field) &&
+		    tracker_string_in_string_list (values[i], old_contents) > -1) {
 			continue;
 		}
 
-		tracker_data_update_set_metadata (service, service_id, field, values[i], NULL);
+		tracker_data_update_set_metadata (context, 
+						  service, 
+						  service_id, 
+						  field, 
+						  values[i], 
+						  NULL);
 		set_values [++j] = values[i];
 	}
 
+	tracker_data_update_metadata_context_close (context);
+	tracker_data_update_metadata_context_free (context);
+
 	joined = g_strjoinv (" ", set_values);
 	if (tracker_field_get_filtered (field)) {
 		index_text_no_parsing (indexer,
@@ -2347,6 +2466,7 @@ handle_metadata_remove (TrackerIndexer *indexer,
 	TrackerField *field;
 	guint service_id, i;
 	gchar *joined = NULL, *dirname = NULL, *basename = NULL;
+	gboolean exists, enabled;
 
 	check_started (indexer);
 
@@ -2381,12 +2501,14 @@ handle_metadata_remove (TrackerIndexer *indexer,
 
 	tracker_file_get_path_and_name (uri, &dirname, &basename);
 
-	tracker_data_query_service_exists (service, dirname, basename, &service_id, NULL);
+	exists = tracker_data_query_service_exists (service,
+						    dirname, basename,
+						    &service_id, NULL, &enabled);
 
 	g_free (dirname);
 	g_free (basename);
 
-	if (service_id < 1) {
+	if (!exists || !enabled) {
 		g_set_error (error,
 			     g_quark_from_string (TRACKER_INDEXER_ERROR),
 			     TRACKER_INDEXER_ERROR_CODE,
@@ -2445,14 +2567,15 @@ handle_metadata_remove (TrackerIndexer *indexer,
 
 static gboolean
 should_change_index_for_file (TrackerIndexer *indexer,
-			      PathInfo        *info,
-			      const gchar	  *dirname,
-			      const gchar	  *basename)
+			      PathInfo       *info,
+			      const gchar    *dirname,
+			      const gchar    *basename)
 {
 	TrackerService *service;
 	gchar *path;
 	guint64 current_mtime;
 	time_t db_mtime;
+	gboolean enabled;
 
 	service = get_service_for_file (info->module_file, info->module);
 
@@ -2467,7 +2590,16 @@ should_change_index_for_file (TrackerIndexer *indexer,
 						dirname,
 						basename,
 						NULL,
-						&db_mtime)) {
+						&db_mtime,
+						&enabled)) {
+		return TRUE;
+	}
+
+	/* Service exists, but is disabled. In this case
+	 * we want to reindex, so the item will be deleted
+	 * afterwards before inserting the new data.
+	 */
+	if (!enabled) {
 		return TRUE;
 	}
 
@@ -2902,12 +3034,16 @@ static gchar *
 state_to_string (TrackerIndexerState state)
 {
 	GString *s;
+	gchar   *str, *p;
 
 	s = g_string_new ("");
 	
 	if (state & TRACKER_INDEXER_STATE_INDEX_OVERLOADED) {
 		s = g_string_append (s, "INDEX_OVERLOADED | ");
 	}
+	if (state & TRACKER_INDEXER_STATE_INDEX_ERROR) {
+		s = g_string_append (s, "INDEX_ERROR | ");
+	}
 	if (state & TRACKER_INDEXER_STATE_PAUSED) {
 		s = g_string_append (s, "PAUSED | ");
 	}
@@ -2918,9 +3054,19 @@ state_to_string (TrackerIndexerState state)
 		s = g_string_append (s, "CLEANUP | ");
 	}
 
-	s->str[s->len - 3] = '\0';
+	str = g_string_free (s, FALSE);
 
-	return g_string_free (s, FALSE);
+	/* Remove last separator */
+	p = g_utf8_strrchr (str, -1, '|');
+	if (p) {
+		/* Go back one to the space before '|' */
+		p--;
+		
+		/* NULL terminate here */
+		*p = '\0';
+	}
+
+	return str;
 }
 
 static void
@@ -3323,9 +3469,11 @@ tracker_indexer_volume_update_state (TrackerIndexer         *indexer,
 	/* tracker_turtle_process_ttl will be spinning the mainloop, therefore
 	   we can already return the DBus method */
 
+#ifdef ENABLE_TTL_LOADER
 	if (enabled) {
 		tracker_removable_device_load (indexer, path);
 	}
+#endif /* ENABLE_TTL_LOADER */
 }
 
 void
