@@ -53,6 +53,7 @@
 #include "tracker-dbus.h"
 #include "tracker-indexer.h"
 #include "tracker-push.h"
+#include "tracker-indexer-glue.h"
 
 #define ABOUT								  \
 	"Tracker " PACKAGE_VERSION "\n"
@@ -87,7 +88,7 @@ static GOptionEntry  entries[] = {
 	  NULL },
 	{ "process-all", 'p', 0,
 	  G_OPTION_ARG_NONE, &process_all,
-	  N_("Whether to process data from all configured modules to be indexed"),
+	  N_("Index data from all enabled modules"),
 	  NULL },
 	{ "run-forever", 'f', 0,
 	  G_OPTION_ARG_NONE, &run_forever,
@@ -268,6 +269,16 @@ indexer_finished_cb (TrackerIndexer *indexer,
 	}
 }
 
+static void
+daemon_availability_changed_cb (const gchar *name,
+                                gboolean     available,
+                                gpointer     user_data)
+{
+        if (!available) {
+                tracker_indexer_stop (TRACKER_INDEXER (user_data));
+        }
+}
+
 gint
 main (gint argc, gchar *argv[])
 {
@@ -380,10 +391,17 @@ main (gint argc, gchar *argv[])
 	indexer = tracker_indexer_new ();
 
 	/* Make Tracker available for introspection */
-	if (!tracker_dbus_register_object (G_OBJECT (indexer))) {
+	if (!tracker_dbus_register_object (G_OBJECT (indexer),
+                                           &dbus_glib_tracker_indexer_object_info,
+                                           TRACKER_INDEXER_PATH)) {
 		return EXIT_FAILURE;
 	}
 
+        /* Listen for changes in trackerd availability */
+        tracker_dbus_add_name_monitor (TRACKER_DAEMON_SERVICE,
+                                       daemon_availability_changed_cb,
+                                       indexer, NULL);
+
 	/* Create the indexer and run the main loop */
 	g_signal_connect (indexer, "finished",
 			  G_CALLBACK (indexer_finished_cb),
