@@ -34,7 +34,6 @@
 #include <glib.h>
 #include <regex.h>
 #include <gdk/gdkx.h>
-#include <libart_lgpl/art_rgb.h>
 #include <gio/gio.h>
 #include <libgnome/gnome-desktop-item.h>
 #include <libgnomeui/gnome-thumbnail.h>
@@ -909,9 +908,7 @@ tracker_search_stretch_frame_image (GdkPixbuf *frame_image,
 				 gboolean fill_flag)
 {
 	GdkPixbuf * result_pixbuf;
-	guchar * pixels_ptr;
 	gint frame_width, frame_height;
-	gint y, row_stride;
 	gint target_width, target_frame_width;
 	gint target_height, target_frame_height;
 
@@ -923,15 +920,10 @@ tracker_search_stretch_frame_image (GdkPixbuf *frame_image,
 	} else {
 		result_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, dest_width, dest_height);
 	}
-	row_stride = gdk_pixbuf_get_rowstride (result_pixbuf);
-	pixels_ptr = gdk_pixbuf_get_pixels (result_pixbuf);
 
 	/* clear the new pixbuf */
 	if (fill_flag == FALSE) {
-		for (y = 0; y < dest_height; y++) {
-			art_rgb_run_alpha (pixels_ptr, 255, 255, 255, 255, dest_width);
-			pixels_ptr += row_stride;
-		}
+		gdk_pixbuf_fill (result_pixbuf, 0xffffffff);
 	}
 
 	target_width  = dest_width - left_offset - right_offset;
