@@ -164,17 +164,6 @@ static void crawler_finished_cb		    (TrackerCrawler   *crawler,
 					     guint	       files_ignored,
 					     gpointer	       user_data);
 
-#ifdef HAVE_HAL
-static void mount_point_added_cb	    (TrackerHal       *hal,
-					     const gchar      *volume_uuid,
-					     const gchar      *mount_point,
-					     gpointer	       user_data);
-static void mount_point_removed_cb	    (TrackerHal       *hal,
-					     const gchar      *volume_uuid,
-					     const gchar      *mount_point,
-					     gpointer	       user_data);
-#endif /* HAVE_HAL */
-
 static guint signals[LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (TrackerProcessor, tracker_processor, G_TYPE_OBJECT)
@@ -334,13 +323,6 @@ tracker_processor_finalize (GObject *object)
 	}
 
 	if (priv->hal) {
-		g_signal_handlers_disconnect_by_func (priv->hal,
-						      mount_point_added_cb,
-						      object);
-		g_signal_handlers_disconnect_by_func (priv->hal,
-						      mount_point_removed_cb,
-						      object);
-
 		g_object_unref (priv->hal);
 	}
 #endif /* HAVE_HAL */
@@ -1242,14 +1224,13 @@ processor_files_check (TrackerProcessor *processor,
 		 is_directory ? "DIR" : "FILE");
 
 	if (!ignored) {
-		if (!is_directory) {
-			queue = g_hash_table_lookup (processor->private->items_created_queues, module_name);
-			g_queue_push_tail (queue, g_object_ref (file));
-			
-			item_queue_handlers_set_up (processor);
-		} else {
+		if (is_directory) {
 			tracker_crawler_add_unexpected_path (crawler, path);
 		}
+		queue = g_hash_table_lookup (processor->private->items_created_queues, module_name);
+		g_queue_push_tail (queue, g_object_ref (file));
+			
+		item_queue_handlers_set_up (processor);
 	}
 
 	g_free (path);
@@ -1525,29 +1506,39 @@ crawler_finished_cb (TrackerCrawler *crawler,
 
 #ifdef HAVE_HAL
 
-static void
-mount_point_added_cb (TrackerHal  *hal,
-		      const gchar *udi,
-		      const gchar *mount_point,
-		      gpointer	   user_data)
+static gchar *
+normalize_mount_point (const gchar *mount_point)
+{
+	if (g_str_has_suffix (mount_point, G_DIR_SEPARATOR_S)) {
+		return g_strdup (mount_point);
+	} else {
+		return g_strconcat (mount_point, G_DIR_SEPARATOR_S, NULL);
+	}
+}
+
+void
+tracker_processor_mount_point_added (TrackerProcessor *processor,
+				     const gchar      *udi,
+				     const gchar      *mount_point)
 {
-	TrackerProcessor        *processor;
 	TrackerProcessorPrivate *priv;
 	TrackerStatus	         status;
 	GList                   *l;
+	gchar                   *mp;
+
+	g_return_if_fail (TRACKER_IS_PROCESSOR (processor));
 
-	processor = user_data;
 	priv = processor->private;
 
 	status = tracker_status_get ();
+	mp = normalize_mount_point (mount_point);
 
 	/* Add removable device to list of known devices to iterate */
-	priv->removable_devices = g_list_append (priv->removable_devices, 
-						 g_strdup (mount_point));
+	priv->removable_devices = g_list_append (priv->removable_devices, mp);
 
 	/* Remove from completed list so we don't ignore it */
-	l = g_list_find_custom (priv->removable_devices_completed, 
-				mount_point,
+	l = g_list_find_custom (priv->removable_devices_completed,
+				mp,
 				(GCompareFunc) g_strcmp0);
 
 	if (l) {
@@ -1569,29 +1560,30 @@ mount_point_added_cb (TrackerHal  *hal,
 		 * crawled all locations so we need to start up the
 		 * processor again for the removable media once more.
 		 */
-		process_module_next (processor);
+		tracker_processor_start (processor);
 	}
 }
 
-static void
-mount_point_removed_cb (TrackerHal  *hal,
-			const gchar *udi,
-			const gchar *mount_point,
-			gpointer     user_data)
+void
+tracker_processor_mount_point_removed (TrackerProcessor *processor,
+				       const gchar      *udi,
+				       const gchar      *mount_point)
 {
-	TrackerProcessor        *processor;
 	TrackerProcessorPrivate *priv;
 	GFile		        *file;
 	GList                   *l;
+	gchar                   *mp;
+
+	g_return_if_fail (TRACKER_IS_PROCESSOR (processor));
 
-	processor = user_data;
 	priv = processor->private;
+	mp = normalize_mount_point (mount_point);
 
 	/* Remove directory from list of iterated_removable_media, so
 	 * we don't traverse it.
 	 */
-	l = g_list_find_custom (priv->removable_devices, 
-				mount_point,
+	l = g_list_find_custom (priv->removable_devices,
+				mp,
 				(GCompareFunc) g_strcmp0);
 
 	if (l) {
@@ -1601,7 +1593,7 @@ mount_point_removed_cb (TrackerHal  *hal,
 	}
 
 	l = g_list_find_custom (priv->removable_devices_completed, 
-				mount_point,
+				mp,
 				(GCompareFunc) g_strcmp0);
 
 	if (l) {
@@ -1616,6 +1608,8 @@ mount_point_removed_cb (TrackerHal  *hal,
 	file = g_file_new_for_path (mount_point);
 	tracker_monitor_remove_recursively (priv->monitor, file);
 	g_object_unref (file);
+
+	g_free (mp);
 }
 
 #endif /* HAVE_HAL */
@@ -1651,13 +1645,6 @@ tracker_processor_new (TrackerConfig *config,
 	priv->removable_devices = tracker_hal_get_removable_device_roots (priv->hal);
 	priv->removable_devices_current = priv->removable_devices;
 	priv->removable_devices_completed = NULL;
-
-	g_signal_connect (priv->hal, "mount-point-added",
-			  G_CALLBACK (mount_point_added_cb),
-			  processor);
-	g_signal_connect (priv->hal, "mount-point-removed",
-			  G_CALLBACK (mount_point_removed_cb),
-			  processor);
 #endif /* HAVE_HAL */
 
 	/* Set up the crawlers now we have config and hal */
@@ -1731,6 +1718,7 @@ tracker_processor_start (TrackerProcessor *processor)
 	processor->private->timer = g_timer_new ();
 
 	processor->private->interrupted = TRUE;
+	processor->private->finished = FALSE;
 
 	process_module_next (processor);
 }
@@ -1742,6 +1730,10 @@ tracker_processor_stop (TrackerProcessor *processor)
 
 	g_return_if_fail (TRACKER_IS_PROCESSOR (processor));
 
+	if (processor->private->finished) {
+		return;
+	}
+
 	if (processor->private->interrupted) {
 		TrackerCrawler *crawler;
 
@@ -1751,7 +1743,6 @@ tracker_processor_stop (TrackerProcessor *processor)
 
 	}
 
-
 	/* Now we have finished crawling, we enable monitor events */
 	g_message ("Enabling monitor events");
 	tracker_monitor_set_enabled (processor->private->monitor, TRUE);
